use bcup::models::{Record, ServerConfiguration};
use futures::StreamExt;
use surrealdb::{
    engine::remote::ws::{Client, Ws},
    opt::PatchOp,
    Notification, Result, Surreal,
};

#[tokio::main]
async fn main() -> surrealdb::Result<()> {
    let db = Surreal::new::<Ws>("127.0.0.1:8000").await?;

    db.use_ns("test").use_db("test").await?;

    println!("Prepared to listen");
    let mut stream = db.select("rest_server").live().await?;
    while let Some(result) = stream.next().await {
        handle(result, &db).await;
    }

    Ok(())
}
async fn handle(result: Result<Notification<ServerConfiguration>>, db: &Surreal<Client>) {
    println!("Something to handle");
    match result {
        Ok(notification) => {
            let conf = &notification.data;
            if conf.state.is_pending() {
                process_new_item(conf, db).await;
            }
        }
        Err(error) => eprintln!("{error}"),
    }
}

async fn process_new_item(conf: &ServerConfiguration, db: &Surreal<Client>) {
    println!("The following item has to be processed {:?}", conf);
    spawn_restic_container(conf.directory.clone());
    add_proxy_configuration(conf.directory.clone());
    mark_data_as_processed(conf, db).await;
}

async fn mark_data_as_processed(conf: &ServerConfiguration, db: &Surreal<Client>) {
    println!("Updating status on item id:{:?}", conf.id);
    let updated: Option<Record> = db
        .update(&conf.id)
        .patch(PatchOp::replace("/state/now", "processed"))
        .await
        .unwrap();
    dbg!(updated);
}

fn spawn_restic_container(path: String) {
    println!("[fake] Spawning container for path {:?}", path);
}
fn add_proxy_configuration(path: String) {
    println!("[fake] Creating caddy configuration {:?}", path);
}
